require "responding/version"

class Responding
  def self.new(*attributes)
    raise ArgumentError.new('At least one attribute required') if attributes.empty?

    ::Class.new(Class) do
      attributes.each do |attribute|
        define_attribute attribute
      end
    end
  end

  class Class
    def initialize(hash = {})
      hash.each do |attr, value|
        public_send :"#{attr}=", value
      end
    end

    private
    def self.define_attribute(attribute)
      case attribute
      when Symbol
        attr_accessor attribute
      when Hash
        attribute.each do |name, nested_attributes|
          nested_cls = Responding.new(*nested_attributes)

          define_method :"#{name}" do
            var_name = :"@#{name}"
            result = instance_variable_get(var_name)
            unless result
              result = nested_cls.new
              instance_variable_set var_name, result
            end
            result
          end

          define_method :"#{name}=" do |value|
            instance_variable_set :"@#{name}", nested_cls.new(value)
          end
        end
      end
    end
  end
end
